@@ -4,7 +4,7 @@
 #' replication of results, regardless of future backend used.
 #' Analogously to `mapply()`, `future_mapply()` is a multivariate version of
 #' `future_sapply()`.
-#' It applies `FUN` to the first elements of each `\ldots` argument,
+#' It applies `FUN` to the first elements of each \ldots argument,
 #' the second elements, the third elements, and so on.
 #' Arguments are recycled if necessary.
 #' 
@@ -12,8 +12,8 @@
 #' 
 #' @param FUN  A function to apply, found via [base::match.fun()].
 #' 
-#' @param \ldots  Arguments to vectorize over (vectors or lists of strictly
-#' positive length, or all of zero length).
+#' @param \ldots  Arguments to vectorize over, will be recycled to common
+#' length, or zero if one of them is of length zero.
 #'
 #' @param MoreArgs  A list of other arguments to `FUN`.
 #'
@@ -21,7 +21,7 @@
 #' result to a vector, matrix or higher dimensional array; see the simplify
 #' argument of [base::sapply()].
 #' 
-#' @param USE.NAMES A logical; use names if the first `\ldots` argument has
+#' @param USE.NAMES A logical; use names if the first \ldots argument has
 #' names, or if it is a character vector, use that character vector as the
 #' names.
 #'
@@ -42,7 +42,7 @@
 #' @keywords manip programming iteration
 #'
 #' @export
-future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE, future.stdout = TRUE, future.conditions = NULL, future.globals = TRUE, future.packages = NULL, future.lazy = FALSE, future.seed = FALSE, future.scheduling = 1.0, future.chunk.size = NULL, future.label = "future_mapply-%d") {
+future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE, future.envir = parent.frame(), future.stdout = TRUE, future.conditions = "condition", future.globals = TRUE, future.packages = NULL, future.seed = FALSE, future.scheduling = 1.0, future.chunk.size = NULL, future.label = "future_mapply-%d") {
   fcn_name <- "future_mapply"
   args_name <- "..."
   
@@ -56,12 +56,17 @@ future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES
   ns <- lengths(dots)
   
   ## Nothing to do?
-  if (all(ns == 0L)) {
+  ## "max-or-0-if-any" recycling rule was introduced in R (>= 4.2.0)
+  if (any(ns == 0L)) {
     if (!USE.NAMES) return(list())
     values <- list()
     first <- dots[[1]]
     names <- names(first)
-    if (is.null(names) && is.character(first)) names <- first
+    if (length(names) > 0L) {
+      names <- character(0L)
+    } else if (is.null(names) && is.character(first)) {
+      names <- character(0L)
+    }
     names(values) <- names
     return(values)
   }
@@ -79,29 +84,50 @@ future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES
   
   stop_if_not(is.null(MoreArgs) || is.list(MoreArgs))
 
-  debug <- getOption("future.debug", FALSE)
-  
-  if (debug) mdebugf("%s() ...", fcn_name)
+  debug <- isTRUE(getOption("future.debug"))
+  debug <- isTRUE(getOption("future.apply.debug", debug))
+  if (debug) {
+    mdebugf_push("%s() ...", fcn_name)
+    on.exit(mdebug_pop())
+  }
 
   ## NOTE TO SELF: We'd ideally have a 'future.envir' argument also for
   ## this function, cf. future().  However, it's not yet clear to me how
   ## to do this, because we need to have globalsOf() to search for globals
   ## from the current environment in order to identify the globals of 
   ## arguments 'FUN' and '...'. /HB 2017-03-10
-  future.envir <- environment()  ## Not used; just to clarify the above.
-  
   envir <- future.envir
-  
+  envir <- environment()
+
+
+  ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  ## Support %globals%, %packages%, %seed%, ...
+  ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  opts <- getOption("future.disposable", NULL)
+  if (length(opts) > 0) {
+    for (name in names(opts)) {
+      var <- sprintf("future.%s", name)
+      assign(var, opts[[name]], envir = environment(), inherits = FALSE)
+    }
+    if (!identical(attr(opts, "dispose"), FALSE)) {
+      options(future.disposable = NULL)
+    }
+  }
+
+
   ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   ## Future expression
   ## - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  ...future.FUN <- NULL ## To please R CMD check
+  
+  ## Set .Random.seed?
   if (is.null(future.seed) || isFALSE(future.seed) || isNA(future.seed)) {
-     expr <- bquote({
+     expr <- quote({
        args <- c(list(FUN = ...future.FUN), ...future.elements_ii, MoreArgs = list(MoreArgs), SIMPLIFY = FALSE, USE.NAMES = FALSE)
        do.call(mapply, args = args)
      })
   } else {
-    expr <- bquote({
+    expr <- quote({
       ...future.FUN2 <- function(..., ...future.seeds_ii_jj) {
         assign(".Random.seed", ...future.seeds_ii_jj, envir = globalenv(), inherits = FALSE)
         ...future.FUN(...)
@@ -120,9 +146,10 @@ future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES
     nX = nX,
     chunk_args = dots,
     MoreArgs = MoreArgs,
-    get_chunk = function(X, chunk) lapply(X, FUN = `[`, chunk),
+    get_chunk = function(X, chunk) lapply(X, FUN = `chunkWith[[`, chunk),
     expr = expr,
     envir = envir,
+    future.envir = future.envir,
     future.globals = future.globals,
     future.packages = future.packages,
     future.scheduling = future.scheduling,
@@ -130,7 +157,6 @@ future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES
     future.stdout = future.stdout,
     future.conditions = future.conditions,
     future.seed = future.seed,
-    future.lazy = future.lazy,
     future.label = future.label,
     fcn_name = fcn_name,
     args_name = args_name,
@@ -153,8 +179,6 @@ future_mapply <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES
     values <- simplify2array(values, higher = (SIMPLIFY == "array"))
   } 
   
-  if (debug) mdebugf("%s() ... DONE", fcn_name)
-  
   values
 }
 
